@@ -6,7 +6,7 @@ Certain services related stuff
 import enum
 # pylint: disable=import-self
 from . import (
-    anti_captcha, azcaptcha, captcha_guru, cptch_net, rucaptcha, twocaptcha
+    anti_captcha, azcaptcha, captcha_guru, cptch_net, deathbycaptcha, rucaptcha, twocaptcha
 )
 
 
@@ -17,6 +17,7 @@ class CaptchaSolvingService(enum.Enum):
     AZCAPTCHA = "azcaptcha.com"
     CAPTCHA_GURU = "captcha.guru"
     CPTCH_NET = "cptch.net"
+    DEATHBYCAPTCHA = "deathbycaptcha.com"
     RUCAPTCHA = "rucaptcha.com"
     TWOCAPTCHA = "2captcha.com"
 
@@ -27,6 +28,7 @@ SOLVING_SERVICE = {
     CaptchaSolvingService.AZCAPTCHA: azcaptcha,
     CaptchaSolvingService.CAPTCHA_GURU: captcha_guru,
     CaptchaSolvingService.CPTCH_NET: cptch_net,
+    CaptchaSolvingService.DEATHBYCAPTCHA: deathbycaptcha,
     CaptchaSolvingService.RUCAPTCHA: rucaptcha,
     CaptchaSolvingService.TWOCAPTCHA: twocaptcha
 }
